/*************************** Includes ********************************/
#include <tjpbase.h>
#include <stdio.h>
#include <algconst.h>
/************************ End of Includes ****************************/

/*************************** Constants ********************************/
#define TABLE_LENGTH 400
#define PAV_TURN	0x40
#define PAV_STRAIGHT 0x80
#define PAV_AFTER 0xFF

/************************ End of Constants ****************************/
#pragma interrupt_handler PAV_isr
/*************************** Prototypes *********************************/
void STRAIGHTEN(void);
void init_PA(void);
void PAV_isr(void);
void TURN_RIGHT(void);
void TURN_LEFT(void);
void TURN_AROUND(void);
void PA_ON(void);
void PA_OFF(void);
/************************ End of Prototypes *****************************/

int counter;
int speedr;
int speedl;
int done;


void main(void)
/****************************** Main ***********************************/
{
	int irdr, irdl, irdc;
	int std;
	int diff;
	
	
	init_analog();
	init_motortjp();
	init_clocktjp();
	init_PA();

	IRE_ON;

	speedr = MAX_RIGHT;
	speedl = MAX_LEFT;
	
	counter=0;

	START;
	
	while(1)
	{

		std = (RIGHT_IR + LEFT_IR) / 2;

		irdl=LEFT_IR;
		irdr=RIGHT_IR;
		irdc=CENTER_IR;

		if ((irdr <= NOWALL) & ((irdc >= STOP) | (irdl >= WALL)))
		{
			TURN_RIGHT();
			STRAIGHTEN();
			STRAIGHTEN();
			STRAIGHTEN();
			STRAIGHTEN();
		}
		else if ((irdl <= NOWALL) & ((irdc >= STOP) | (irdr >= WALL)))
		{
			TURN_LEFT();
			STRAIGHTEN();
			STRAIGHTEN();
			STRAIGHTEN();
			STRAIGHTEN();
		}
		else if ((irdc >= STOP) & ((irdr >= WALL) & (irdl >= WALL)))
		{
			TURN_AROUND();
			STRAIGHTEN();
			STRAIGHTEN();
			STRAIGHTEN();
			STRAIGHTEN();
		}
	
		if((irdr < IR_MIN+1) & (irdc < IR_MIN+1) & (irdl < IR_MIN+1))
		{
			motorp(RIGHT_MOTOR,0);
			motorp(LEFT_MOTOR,0);
			speedr=MAX_RIGHT;
			speedl=MAX_LEFT;
			START;
		}			 

		if(FRONT_BUMP)
		{
			speedr=-MAX_RIGHT;
			speedl=-MAX_LEFT;
			motorp(RIGHT_MOTOR, speedr);
			motorp(LEFT_MOTOR, speedl);
			wait(5 * DELAY_TIME);
			counter=TABLE_LENGTH-1;
		}
				
		motorp(RIGHT_MOTOR, speedr);
		motorp(LEFT_MOTOR, speedl);
		wait(DELAY_TIME);
	}
}
	

void TURN_RIGHT(void)
{
			PA_ON();
			counter=1;
			PACNT=PAV_STRAIGHT;
			done=0;
			
			motorp(RIGHT_MOTOR, MAX_RIGHT);
			motorp(LEFT_MOTOR, MAX_LEFT);
			while(done==0)
			{
				asm("NOP");
			}
			counter=1;
//			PACNT = PAV_TURN + 5;			
			PACNT = PAV_TURN;
			done=0;
			motorp(RIGHT_MOTOR, 0);
			motorp(LEFT_MOTOR, MAX_LEFT);
			while(done==0)
			{
				asm("NOP");
			}

			counter=1;
			PACNT = PAV_AFTER;
			done=0;			
			motorp(RIGHT_MOTOR, MAX_RIGHT);
			motorp(LEFT_MOTOR, MAX_LEFT);
			while(done==0)
			{
				asm("NOP");
			}

			motorp(RIGHT_MOTOR, 0);
			motorp(LEFT_MOTOR, 0);			
			PA_OFF();
}



void TURN_LEFT(void)
{
			PA_ON();
			counter=1;
			PACNT=PAV_STRAIGHT;
			done=0;
			
			motorp(RIGHT_MOTOR, MAX_RIGHT);
			motorp(LEFT_MOTOR, MAX_LEFT);
			while(done==0)
			{
				asm("NOP");
			}

			counter=1;
			PACNT = PAV_TURN;			
			done=0;
			motorp(RIGHT_MOTOR, MAX_RIGHT);
			motorp(LEFT_MOTOR, 0);
			while(done==0)
			{
				asm("NOP");
			}
			
			counter=1;
			PACNT = PAV_AFTER;
			done=0;			
			motorp(RIGHT_MOTOR, MAX_RIGHT);
			motorp(LEFT_MOTOR, MAX_LEFT);
			while(done==0)
			{
				asm("NOP");
			}
		
			motorp(RIGHT_MOTOR, 0);
			motorp(LEFT_MOTOR, 0);			
			PA_OFF();
}


void PAV_isr(void)
{
	if(counter==0)
	{
		motorp(RIGHT_MOTOR,0);
		motorp(LEFT_MOTOR,0);
		done=1;
	}
	else
	{
		counter--;
	}
	CLEAR_FLAG(TFLG2, 0x20);
}

void init_PA(void)
{
	INTR_OFF();

	CLEAR_BIT(PACTL, 0xB0); /*Set for input, counting, and catch falling edges*/
	CLEAR_BIT(PACTL, 0x40); /* Disable PA until needed */
	

	CLEAR_FLAG(TFLG2, 0x20); /* Clear flag */
	SET_BIT(TMSK2, 0x20); /* Enable PA Overflow interrupt */

	PACNT=0x00;

	INTR_ON();
}


void TURN_AROUND(void)
{
	motorp(RIGHT_MOTOR, -MAX_RIGHT);
	motorp(LEFT_MOTOR, MAX_LEFT);
	wait(TURN_AROUND_DELAY);
	speedr = MAX_RIGHT;
	speedl = MAX_LEFT;
}

void PA_ON(void)
{
	INTR_OFF();
	SET_BIT(PACTL, 0x40); // Enable PA
	INTR_ON();
}

void PA_OFF(void)
{
	INTR_OFF();
	CLEAR_BIT(PACTL, 0x40); // Disable PA
	INTR_ON();
}

void STRAIGHTEN(void)
{

	int irdr, irdl, irdc;
	int diff, std;
	
//	std=(RIGHT_IR + LEFT_IR) / 2;

	std=125;

	irdl=LEFT_IR;
	irdr=RIGHT_IR;
	irdc=CENTER_IR;
		
	if (irdr > irdl)
	{
		diff = ((irdr - irdl) * AMOUNT) / (IR_MAX - std);
		speedl = MAX_LEFT - diff;
	}
	
	if (irdl > irdr)
	{
		diff = ((irdl - irdr) * AMOUNT) / (IR_MAX - std);
		speedr = MAX_RIGHT - diff;
	}
			
	motorp(RIGHT_MOTOR, speedr);
	motorp(LEFT_MOTOR, speedl);
	wait(DELAY_TIME);
}

