/************************************************************************
 * 			MEKATRONIX Copyright 1998 		        *
 * Title        avoidtjp.c                                              *
 * Programmer	Keith L. Doty and modified for IC by Scott Jantz     	*
 * Date         August 25, 1998                                         *
 * Version	1			                		*
 *								        *
 * Description								*
 *    A very simple collision avoidance program. TJ PRO will read 	*
 *    each IR detectors, and turn away from any obstacles in its	*
 *    path. Also, if something hits TJ PRO's bumper, it will back up,	*
 *    turn, and go on.							*
 *			                                                *     					*
 ************************************************************************/
 

/*************************** Includes ********************************/

#include <tjpbase.h>
#include <stdio.h>

/************************ End of Includes ****************************/


/*************************** Constants ********************************/

#define AVOID_THRESHOLD 100
#define IR_MAX 126
#define IR_MIN 88
#define MAX_RIGHT 75
#define MAX_LEFT 100
/************************ End of Constants ****************************/
#pragma interrupt_handler PAV_isr


/*************************** Prototypes *********************************/
void init_PA(void);
void PAV_isr(void);
/************************ End of Prototypes *****************************/

int counter;
int speedr;
int speedl;

void main(void)
/****************************** Main ***********************************/
{
	int irdr, irdl, stdr, stdl;
	int upr, upl, downr, downl;
	int diff;

	init_analog();
	init_motortjp();
	init_clocktjp();

	START;

	IRE_ON;
	
	stdr=100;
	stdl=100;
	
	upr = IR_MAX - stdr;
	downr = stdr - IR_MIN;
	
	upl = IR_MAX - stdl;
	downl = stdl - IR_MIN;
	
	
	
	while(1)
	{
		
		speedr=MAX_RIGHT;
		speedl=MAX_LEFT;
	
		irdr=RIGHT_IR;
		irdl=LEFT_IR;
		
		if (irdr > stdr)
		{
			diff = ((irdr-stdr) * 100) / upr;
			speedl=MAX_LEFT-diff;
		}
		
		else
		{
			diff = ((stdr - irdr) * 1030) / upr;
			speedr=MAX_RIGHT - diff;
		}		

		if ((irdr < IR_MIN + 3) & (irdl < IR_MIN+3))
		{
			speedr=0;
			speedl=0;
		}
				
		motorp(RIGHT_MOTOR, speedr);
		motorp(LEFT_MOTOR, speedl);
		wait(50);
	}
}
	

void PAV_isr(void)
{
	counter=counter+1;
	if (counter==10)
	{
	 speedr=0;
	 speedl=0;
	}
	CLEAR_FLAG(TFLG2, 0x20);
}

void init_PA(void)
{
	INTR_OFF();

	CLEAR_BIT(PACTL, 0xB0); /*Set for input, counting, and catch falling edges*/
	SET_BIT(PACTL, 0x40); /*Enable PA*/
	

	CLEAR_FLAG(TFLG2, 0x20); /* Clear flag */
	SET_BIT(TMSK2, 0x20); /* Enable PA Overflow interrupt */

	PACNT=0x00;

	INTR_ON();
}
