/*************************** Includes ********************************/
#include <tjpbase.h>
#include <stdio.h>
#include <algconst.h>
/************************ End of Includes ****************************/

/*************************** Constants ********************************/
#define TABLE_LENGTH 400

/************************ End of Constants ****************************/
#pragma interrupt_handler PAV_isr
/*************************** Prototypes *********************************/
void init_PA(void);
void PAV_isr(void);
void TURN_RIGHT(void);
void TURN_LEFT(void);
void TURN_AROUND(void);
/************************ End of Prototypes *****************************/

int speedr, speedl;

void main(void)
/****************************** Main ***********************************/
{
	int irdr, irdl, irdc;
	int std;
	int diff;

	init_analog();
	init_motortjp();
	init_clocktjp();

	IRE_ON;

	speedr = MAX_RIGHT;
	speedl = MAX_LEFT;
	
	std = (RIGHT_IR + LEFT_IR) / 2;

	START;

	while(1)
	{
		
		irdl=LEFT_IR;
		irdr=RIGHT_IR;
		irdc=CENTER_IR;
		
		if (irdr > irdl)
		{
			diff = ((irdr - irdl) * AMOUNT) / (IR_MAX - std);
			speedl = MAX_LEFT - diff;
		}
		
		if (irdl > irdr)
		{
			diff = ((irdl - irdr) * AMOUNT) / (IR_MAX - std);
			speedr = MAX_RIGHT - diff;
		}

		if((irdc >= STOP) & (irdr >= WALL & irdl <= NOWALL))
		{
			TURN_LEFT();
		}
		
		if ((irdc >= STOP) & (irdr >= WALL) & (irdl >= WALL))
		 {
		 	TURN_AROUND();
		 }
		 
		if((irdr < IR_MIN) & (irdc < IR_MIN) & (irdl < IR_MIN))
		{
			motorp(RIGHT_MOTOR,0);
			motorp(LEFT_MOTOR,0);
			speedr=MAX_RIGHT;
			speedl=MAX_LEFT;
			START;
		}			 

		if(FRONT_BUMP)
		{
			speedr=-MAX_RIGHT;
			speedl=-MAX_LEFT;
			motorp(RIGHT_MOTOR, speedr);
			motorp(LEFT_MOTOR, speedl);
			wait(10 * DELAY_TIME);
			
		}
				
		motorp(RIGHT_MOTOR, speedr);
		motorp(LEFT_MOTOR, speedl);
		wait(DELAY_TIME);
	}
}
	

void PAV_isr(void)
{
	CLEAR_FLAG(TFLG2, 0x20);
}

void init_PA(void)
{
	INTR_OFF();

	CLEAR_BIT(PACTL, 0xB0); /*Set for input, counting, and catch falling edges*/
	SET_BIT(PACTL, 0x40); /*Enable PA*/
	

	CLEAR_FLAG(TFLG2, 0x20); /* Clear flag */
	SET_BIT(TMSK2, 0x20); /* Enable PA Overflow interrupt */

	PACNT=0x00;

	INTR_ON();
}

void TURN_RIGHT(void)
{
	motorp(RIGHT_MOTOR, MAX_RIGHT);
	motorp(LEFT_MOTOR, MAX_LEFT);
	wait(TURN_DELAY/2);

	motorp(RIGHT_MOTOR, 0);
	motorp(LEFT_MOTOR, MAX_LEFT);
	wait(TURN_DELAY);
		
	motorp(RIGHT_MOTOR, MAX_RIGHT);
	motorp(LEFT_MOTOR, MAX_LEFT);
	wait(TURN_DELAY/2);

	speedr=MAX_RIGHT;
	speedl=MAX_LEFT;
}

void TURN_LEFT(void)
{
	motorp(RIGHT_MOTOR, MAX_RIGHT);
	motorp(LEFT_MOTOR, MAX_LEFT);
	wait(TURN_DELAY/2);

	motorp(RIGHT_MOTOR, MAX_RIGHT);
	motorp(LEFT_MOTOR, 0);
	wait(TURN_DELAY);
			
	motorp(RIGHT_MOTOR, MAX_RIGHT);
	motorp(LEFT_MOTOR, MAX_LEFT);
	wait(TURN_DELAY/2);

	speedr=MAX_RIGHT;
	speedl=MAX_LEFT;
}

void TURN_AROUND(void)
{
	motorp(RIGHT_MOTOR, -MAX_RIGHT);
	motorp(LEFT_MOTOR, MAX_LEFT);
	wait(TURN_AROUND_DELAY);
	speedr = MAX_RIGHT;
	speedl = MAX_LEFT;

}