//battery must be fully charged for it to stop

/*************************** Includes ********************************/
#include <tjpbase.h>
#include <stdio.h>
/************************ End of Includes ****************************/

/*************************** Constants ********************************/
#define AVOID_THRESHOLD 100
#define IR_MAX 126
#define IR_MIN 88
#define MAX_RIGHT 75
#define MAX_LEFT 100
#define AMOUNT 300
#define DELAY_TIME 25
#define STOP 123
#define NOWALL 90
#define WALL 105
#define STRAIGHT_DELAY 550
#define TURN_DELAY 850
#define TURN_AROUND_DELAY 800

/************************ End of Constants ****************************/
#pragma interrupt_handler PAV_isr
/*************************** Prototypes *********************************/
void init_PA(void);
void PAV_isr(void);
void TURN_RIGHT(void);
void TURN_LEFT(void);
void TURN_AROUND(void);
void update_LED(void);
/************************ End of Prototypes *****************************/

int speedr, speedl;
int smallcounter, bigcounter;

void main(void)
/****************************** Main ***********************************/
{
	int irdr, irdl, irdc;
	int done;
	int diff;

	init_analog();
	init_motortjp();
	init_clocktjp();

	IRE_ON;

	speedr = MAX_RIGHT;
	speedl = MAX_LEFT;
	
	done=0;
	smallcounter=0;
	bigcounter=0;

	START;
	

	while(1)
	{
		
		speedr=MAX_RIGHT;
		speedl=MAX_LEFT;
	
		irdr=RIGHT_IR;
		irdl=LEFT_IR;
		irdc=CENTER_IR;

		if(irdr >= WALL)
		{
			diff = irdr - WALL;
			
			if(diff >= 1)
			{
			 speedl = MAX_LEFT - 25;
			}
			if(diff >= 2)
			{
			 speedl = MAX_LEFT - 40;
			}
			if(diff >= 4)
			{
			 speedl = MAX_LEFT - 75;
			}
			if(diff >= 6)
			{
			 speedl = MAX_LEFT - 90;
			}
		}
		
		else
		{
			diff = WALL - irdr;
			
			if(diff >= 1)
			{
			 speedr = MAX_RIGHT - 25;
			}
			if(diff >= 2)
			{
			 speedr = MAX_RIGHT - 35;
			}
			if(diff >= 4)
			{
			 speedr = MAX_RIGHT - 55;
			}
			if(diff >= 6)
			{
			 speedr = MAX_RIGHT - 70;
			}
		}
		
		
		if ((irdc >= STOP) & (irdr >= WALL) & (irdl >= WALL))
		{
			TURN_AROUND();
		}
		
		if((irdr < IR_MIN+1) & (irdc < IR_MIN+1) & (irdl < IR_MIN+1))
		{
			motorp(RIGHT_MOTOR,0);
			motorp(LEFT_MOTOR,0);
			speedr=MAX_RIGHT;
			speedl=MAX_LEFT;
			START;
		}			 

		if(FRONT_BUMP)
		{
			speedr=-MAX_RIGHT;
			speedl=-MAX_LEFT;
			motorp(RIGHT_MOTOR, speedr);
			motorp(LEFT_MOTOR, speedl);
			wait(5 * DELAY_TIME);
			
		}
				
		motorp(RIGHT_MOTOR, speedr);
		motorp(LEFT_MOTOR, speedl);
		wait(DELAY_TIME);
	}
}
	

void PAV_isr(void)
{
	CLEAR_FLAG(TFLG2, 0x20);
	smallcounter++;
	if(smallcounter==5)
	{
		smallcounter=0;
		bigcounter++;
		if(bigcounter > 9)
		{
		 bigcounter=0;
		}
		update_LED();
	}
}

void init_PA(void)
{
	INTR_OFF();

	CLEAR_BIT(PACTL, 0xB0); /*Set for input, counting, and catch falling edges*/
	SET_BIT(PACTL, 0x40); /*Enable PA*/
	

	CLEAR_FLAG(TFLG2, 0x20); /* Clear flag */
	SET_BIT(TMSK2, 0x20); /* Enable PA Overflow interrupt */

	PACNT=0x00;

	INTR_ON();
}


void TURN_AROUND(void)
{
	motorp(RIGHT_MOTOR, MAX_RIGHT);
	motorp(LEFT_MOTOR, -MAX_LEFT);
	wait(TURN_AROUND_DELAY);
	speedr = MAX_RIGHT;
	speedl = MAX_LEFT;

}

void update_LED(void)
{
}
