;*****************************************************************************
;                                                                            *
;                     UNSIGNED INTEGER TO FLOATING POINT                     *
;                                                                            *
;        This subroutine performs "unsigned" integer to floating point       *
;        conversion of a 16 bit word.  The 16 bit integer must be in the     *
;        double accumulator D.  The resulting floating point number is       *
;        returned in FPACC1.                                                 *
;                                                                            *
;*****************************************************************************
;
;
	.area	text
uint_to_fp:
         ldx    #0x008E      ; LOAD SIGN AND EXPONENT
SINTFLT1: addd   #0          ; CHECK FOR ZERO AND CHECK NORMALIZATION
         bne    UINTFLT2     ;NOT ZERO
         jmp    __fp_return_zero      ;ZERO - RETURN FLOATING ZERO
UINTFLT1: dex                ; DECREMENT EXPONENT
         lsld                ;MULTIPLY MANTISSA BY 2
UINTFLT2: bpl    UINTFLT1    ; KEEP GOING IF NOT NORMALIZED
         std    __fpacc1_mantissa     ;SAVE MANTISSA
         clr    __fpacc1_mantissa+2   ;MANTISSA LSB IS ALWAYS ZERO
         xgdx                ;GET SIGN AND EXPONENT BYTES
         staa   __fpacc1_mantissa_sign     ;SAVE SIGN BYTE
         stab   __fpacc1_exp     ;SAVE EXPONENT BYTE
         clrb                ;NO ERRORS.
         rts                 ;RETURN.
;
;
;
;*****************************************************************************
;                                                                            *
;                      SIGNED INTEGER TO FLOATING POINT                      *
;                                                                            *
;        This routine works just like the unsigned integer to floating       *
;        point routine except the the 16 bit integer in the double accum-    *
;        ulator D is considered to be in two's complement format.  This      *
;        will return a floating point number in the range -32768 to +32767.  *
;                                                                            *
;*****************************************************************************
;
;
int_to_fp::
         tsta                ;CHECK FOR NEGATIVE INTEGER
         bpl    uint_to_fp     ;NUMBER IS POSITIVE - TREAT IT LIKE UNSIGNED
         coma                ;TAKE TWOS COMPLEMENT TO MAKE POSITIVE
         comb
         addd   #1
         ldx    #0xFF8E      ; LOAD SIGN AND EXPONENT
         bra    SINTFLT1     ;CONTINUE WITH CONVERSION

__int2fp::
	pshx
	jsr int_to_fp
	pulx
	rts
__uint2fp::
	pshx
	jsr uint_to_fp
	pulx
	rts

__long2fp::
	jsr __pshfp2	; save fp2

	pshx
	pshy

	pshx	; allocate stack space
	pshx
	tsy
	jsr __lreg2y	; make an arg
	jsr __c_long2fp
	pulx
	pulx

	puly
	pulx

	jsr	__pulfp2	; restore fp2
	rts

__ulong2fp::
	jsr __pshfp2	; save fp2

	pshx
	pshy

	pshx	; allocate stack space
	pshx
	tsy
	jsr __lreg2y	; make an arg
	jsr __c_ulong2fp
	pulx
	pulx

	puly
	pulx

	jsr	__pulfp2	; restore fp2
	rts

__fp2long::
	pshx

	pshx		; allocate stack
	pshx
	tsy
	jsr __fromfp1	; copy from fpacc1 to arg1 space
	jsr __c_fp2long
	pulx
	pulx

	pulx
	rts
