#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

typedef unsigned int UINT;

int _print(char ** ps, void (*_put) (char **, UINT), char * fmt, va_list va);

extern void itoa (char * buf, unsigned int i, int base);
#ifndef NO_LONG
extern void ltoa (char * buf, unsigned long i, int base);
#endif

#define F_ALTFORM 	0x0001
#define F_LONG 		0x0002
#define F_LEFTALIGN 0x0004
#define F_ADDSIGN 	0x0008
#define F_ZEROPADD 	0x0010

int
_print (char ** ps, void (*_put) (char **, UINT), char * fmt, va_list va)
{
  char buf[25];
  char *bp;
  char *s;
  int i;
  int ret = 0;
  int flags;
  char signch;
  int width;
  int prec;
  int done;
#ifndef NO_LONG
  long l;
#endif

  for ( ; *fmt; fmt++)
	{
      if (*fmt != '%')
		  {
          (*_put) (ps, *fmt), ++ret;
		  continue;
		  }
      bp = buf;
	  flags = 0;
	  width = 0;
	  prec = 0;
	  signch = ' ';
	  ++fmt;

	  /* check for the presence of flag characters: */
	for (done = 0; ; fmt++)
		{
		switch (fmt[0])
			{
		case '#':
			flags |= F_ALTFORM;
			break;
		case '-':
			flags |= F_LEFTALIGN;
			break;
		case '+':
		  	flags |= F_ADDSIGN;
		  	signch = '+';
			break;
		case ' ':
		  	flags |= F_ADDSIGN;
		  	signch = ' ';
			break;
		case '0':
			flags |= F_ZEROPADD;
			break;
		default:
			done = 1;
			break;
			}
		if (done)	/* break here so fmt++ would not happen */
			break;
		}

	  /* read the 'width' and 'precision' fields: */
	  if (isdigit (fmt[0]))
		{
		  width = atoi ((char *) fmt);
		  while (isdigit (*fmt))
			  fmt++;
		}
	  if (fmt[0] == '.')
		{
		  ++fmt;
		  prec = atoi ((char *) fmt);
		  while (isdigit (*fmt))
			  fmt++;
		}
	  if (fmt[0] == 'l')
		{
		  ++fmt;
		  flags |= F_LONG;
		}
	  /* look for the base conversion type: */
	  switch (*fmt)
		{
		default:
		  (*_put) (ps, *fmt), ++ret;
		  break;
		case 'd':
#ifndef NO_LONG
		  if (flags & F_LONG)
			{
			  l = va_arg (va, long);
			  if (l < 0)
				{
				  flags |= F_ADDSIGN;
				  signch = '-';
				  l = -l;
				}
			  ltoa (bp, l, 10);
			}
		  else
#endif
			{
			  i = va_arg (va, int);
			  if (i < 0)
				{
				  flags |= F_ADDSIGN;
				  signch = '-';
				  i = -i;
				}
			  itoa (bp, i, 10);
			}
		  s = buf;
		  goto print_string;
		case 'u':
		case 'o':
		case 'x':
#ifndef NO_LONG
		  if (flags & F_LONG)
			  l = va_arg (va, unsigned long);
		  else
#endif
			  i = va_arg (va, unsigned int);
		  if (*fmt == 'u')
			{
#ifndef NO_LONG
			  if (flags & F_LONG)
				ltoa (bp, l, 10);
			  else
#endif
				itoa (bp, i, 10);
			}
		  else if (*fmt == 'o')
			{
			  if (flags & F_ALTFORM)
				{
				  *bp++ = '0';
				}
#ifndef NO_LONG
			  if (flags & F_LONG)
				  ltoa (bp, l, 8);
			  else
#endif
				  itoa (bp, i, 8);
			}
		  else
			{
			  if (flags & F_ALTFORM)
				{
				  *bp++ = '0';
				  *bp++ = 'x';
				}
#ifndef NO_LONG
			  if (flags & F_LONG)
				  ltoa (bp, l, 16);
			  else
#endif
				  itoa (bp, i, 16);
			}
		  s = buf;
		  goto print_string;
		case 'c':
		  i = va_arg (va, int);
		  (*_put) (ps, i), ++ret;
		  break;
		case 's':
		  s = va_arg (va, char *);
		  if (!s)
			  s = "(null ptr passed)";
		print_string:
		  /* width is the total length of the output desired
		   * prec is the length of the string.
		   * First make width the length of the padding
		   */
		  width -= strlen(s);
		  if (prec == 0)
			prec = strlen(s);
		  if (flags & F_ADDSIGN)
			width--;
		  /* zero padded, 
		   * put sign character, then 0s
		   */
		  if (flags & F_ZEROPADD)
			{
			  if (flags & F_ADDSIGN)
				  (*_put) (ps, signch), ++ret;
			  if (!(flags & F_LEFTALIGN))
				  while (width > 0)
					{
					  (*_put) (ps, '0'), ++ret;
					  width--;
					}
			}
		  /* space padded
		   * pad, then add sign character
		   */
		  else
			{
			  if (!(flags & F_LEFTALIGN))
				  while (width > 0)
					{
					  (*_put) (ps, ' '), ++ret;
					  width--;
					}
			  if (flags & F_ADDSIGN)
				  (*_put) (ps, signch);
			 }
		  /* now put the actual string */
		  while (prec--)
			  (*_put) (ps, *s++), ++ret;
		  /* then pad if necessary */
		  if (flags & F_LEFTALIGN)
			{
			  while (width > 0)
				{
				  (*_put) (ps, ' '), ++ret;
				  width--;
				}
			}
		  break;
		case 'f':
#ifdef FLOAT_PRINTF
		  {
			char *exp, *dot, *p;
			int digits, steps, move_left;
			float f = va_arg (va, float);
			extern char *_fp_buf ();

			s = _fp_buf (f);
			/* fp_to_string does not give format that C users are
			 * used to. Convert.
			 */
			if (s[0] == ' ' || s[0] == '-')
				{
				if (s[0] == '-')
					{
					flags |= F_ADDSIGN;
					signch = '-';
					}
				if (s[1] == '.')
					s[0] = '0';
				else
					s++;
				}
			if ((dot = strchr(s, '.')) == 0)
				{
				dot = &s[strlen(s)];
				dot[0] = '.'; dot[1] = '0'; dot[2] = 0;
				}
			else if (dot[1] == 0)
				{
				dot[1] = '0'; dot[2] = 0;
				}
			/* now get rid of exponent */
			if (exp = strchr(s, 'E'))
				{
				exp[0] = 0;
				if (exp[1] == '-')
					move_left = 1, steps = atoi(exp + 2);
				else
					move_left = 0, steps = atoi(exp + 1);
				p = buf;
				/* move decimal point to left */
				if (move_left)
					{
					digits = dot - s;	/* # of decimal digits */
					/* need to add 0.0... */
					if (digits <= steps)
						{
						*p++ = '0';
						*p++ = '.';
						while (digits < steps)
							*p++ = '0', --steps;
						}
					/* move the decimal point */
					else
						{
						while (digits > steps)
							{
							*p++ = *s++;
							--digits;
							}
						*p++ = '.';
						}
					/* copy all the digits up to the old dot */
					memcpy(p, s, digits);
					p += digits;
					/* copy all the digits after the old dot */
					strcpy(p, dot+1);
					}
				/* move decimal point to right */
				else
					{
					/* copy up to the dot */
					memcpy(p, s, dot-s);
					p += (dot-s);
					/* copy "steps" # of frac. digits, use '0' if necessary */
					for (s = dot+1; steps; --steps)
						if (*s)
							*p++ = *s++;
						else
							*p++ = '0';
					*p++ = '.';
					if (*s)
						strcpy(p, s);
					else
						{
						*p++ = '0';
						*p = 0;
						}
					}
				dot = strchr(buf, '.');
				s = buf;
				}	/* check for 'E' */
			/* now the string looks like <digit>*.<digit>* */
			if (prec)
				{
				p = &dot[strlen(dot)];		/* end of string */
				digits = (p - dot) - 1;		/* # of frac. digits */
				/* pad frac with '0' is needed */
				while (digits < prec)
					*p++ = '0', digits++;
				*p = 0;
				/* truncate frac digits if needed, round up */
				if (digits > prec)
					{
					if ('5' <= dot[prec+1])
						{
						/* if there is no digit less than 9, then don't
						 * increment 
						 */
						for (p = s; p != &dot[prec+1]; p++)
							if ('0' <= *p && *p <= '8')
								break;
						if (p != &dot[prec+1])
							for (p = &dot[prec]; p >= s; --p)
								{
								if (p[0] == '9')
									p[0] = '0';
								else if (*p != '.')
									{
									p[0]++;
									break;
									}
								}
						}
					dot[prec+1] = 0;
					}
				/* print_string takes prec to mean the whole string */
				prec += (dot-s)+1;	/* plus '.' */
				}
		  }
#else
		  s = (char *) "<integer only print, use -lfp?? to relink>";
#endif
		  goto print_string;

		}
    }
  return ret;
}
